library imonke;

import "dart:convert" as convert;
import "dart:io" as io;
import "package:http/http.dart" as http;

String _api = io.Platform.environment["IMONKE_API"];
String _current;
Map<String, Client> _clients = {};

Client getCurrent() => _current == null ? null : getClient(_current);
void setCurrent(String email) => _current = email;

List<String> listClient() => _clients.keys.toList(growable: false);
Client getClient(String email) {
  if (_clients[email] == null) {
    _clients[email] = Client()..email = email;
  }

  return _clients[email];
}

void dropClient(String email) => _clients.remove(email);

class Client {
  String api = _api;
  String _email, _secret, _token;
  BigInt _tokenExpires;

  BigInt get now => BigInt.from(DateTime.now().millisecondsSinceEpoch ~/ 1000);

  String get email => _email;
  String get secret => _secret;
  BigInt get tokenExpires => _tokenExpires;

  void set email(String it) => _email = it;
  void set secret(String it) => _secret = it;

  Client({api}) {
    if (api != null) {
      this.api = api;
    }
  }

  Future<Map<String, String>> get headers async {
    if (_token != null && _tokenExpires >= now) {
      return {"Authorization": "Bearer " + _token};
    }

    if (_secret == null || _email == null) {
      return {};
    }

    String localSecret = _secret;
    _secret = null;

    await login(_email, secret: localSecret);
    return headers;
  }

  Future<Map<String, dynamic>> request(
      String method, route, Map<String, dynamic> data,
      {bool skipHeaders: false}) async {
    http.Request sendable = http.Request(method, Uri.parse(api + route))
      ..headers.addAll(skipHeaders ? {} : await headers)
      ..body = data == null ? null : convert.jsonEncode(data);

    return convert
        .jsonDecode(await (await sendable.send()).stream.bytesToString());
  }

  Future<Client> login(String email, {String password, secret}) async {
    Map<String, dynamic> response = await request(
      "POST",
      "/auth",
      {"email": email, "password": password, "secret": secret},
    );

    if (response["error"] == null) {
      _email = email;
      _secret = response["auth"]["secret"];
      _token = response["auth"]["token"];
      _tokenExpires = BigInt.from(response["auth"]["expires"]);
    }

    return this;
  }
}
