import 'package:test/test.dart';
import 'package:imonke/client.dart' as imonke;

const String email = "mail@gastrodon.io";
const String password = "foobar2000";

void main() {
  test("login", test_login);
  test("login with secret", test_loginSecret);
  test("login bad secret fails", test_loginBadSecret);
  test("get client of email persist", test_getClient);
  test("drop client", test_dropClient);
  test("list client", test_listClient);
  test("get current", test_getCurrent);
}

void test_login() async {
  imonke.Client me = await imonke.Client().login(email, password: password);

  expect(me.email, email);
  expect(me.secret, isNot(""));
  expect(me.tokenExpires, isNot(null));
  expect(me.tokenExpires, isNot(BigInt.from(0)));
}

void test_loginSecret() async {
  imonke.Client me = await imonke.Client().login(email, password: password);

  String secret = me.secret;
  me = await me.login(email, secret: me.secret);

  expect(me.secret, isNot(secret));
  expect(me.secret, isNot(""));
}

void test_loginBadSecret() async {
  imonke.Client me = await imonke.Client().login(email, password: password);

  imonke.Client them = imonke.Client()
    ..secret = me.secret
    ..email = email;

  expect(them.headers, isNot({}));
  expect(them.secret, isNot(me.secret));
  expect(them.secret, isNot(""));
}

void test_getClient() async {
  imonke.Client me =
      await imonke.getClient(email).login(email, password: password);

  expect(me.secret, isNot(""));
  expect(me.secret, imonke.getClient(email).secret);
  expect(me.email, imonke.getClient(email).email);
}

void test_dropClient() async {
  imonke.Client me =
      await imonke.getClient(email).login(email, password: password);

  imonke.dropClient(email);

  imonke.Client them = await imonke.getClient(email);

  expect(me.email, them.email);
  expect(me.secret, isNot(them.secret));
}

void test_listClient() async {
  expect(<String>[email], imonke.listClient());
}

void test_getCurrent() async {
  imonke.setCurrent(email);

  expect(imonke.getCurrent().email, email);
}
